@@ -13,13 +13,12 @@
 // limitations under the License.
 
 use itertools::Itertools;
-use jj_lib::matchers::EverythingMatcher;
 use jj_lib::repo::Repo;
+use jj_lib::revset::{RevsetExpression, RevsetFilterPredicate};
 use jj_lib::rewrite::merge_commit_trees;
 use tracing::instrument;
 
-use super::resolve;
-use crate::cli_util::CommandHelper;
+use crate::cli_util::{print_conflicted_paths, CommandHelper};
 use crate::command_error::CommandError;
 use crate::diff_util;
 use crate::ui::Ui;
@@ -34,13 +33,17 @@ use crate::ui::Ui;
 ///  * Conflicted branches (see https://github.com/martinvonz/jj/blob/main/docs/branches.md)
 #[derive(clap::Args, Clone, Debug)]
 #[command(visible_alias = "st")]
-pub(crate) struct StatusArgs {}
+pub(crate) struct StatusArgs {
+    /// Restrict the status display to these paths
+    #[arg(value_hint = clap::ValueHint::AnyPath)]
+    paths: Vec<String>,
+}
 
 #[instrument(skip_all)]
 pub(crate) fn cmd_status(
     ui: &mut Ui,
     command: &CommandHelper,
-    _args: &StatusArgs,
+    args: &StatusArgs,
 ) -> Result<(), CommandError> {
     let workspace_command = command.workspace_helper(ui)?;
     let repo = workspace_command.repo();
@@ -48,6 +51,9 @@ pub(crate) fn cmd_status(
         .get_wc_commit_id()
         .map(|id| repo.store().get_commit(id))
         .transpose()?;
+    let matcher = workspace_command
+        .parse_file_patterns(&args.paths)?
+        .to_matcher();
     ui.request_pager();
     let mut formatter = ui.stdout_formatter();
     let formatter = formatter.as_mut();
@@ -62,17 +68,19 @@ pub(crate) fn cmd_status(
             diff_util::show_diff_summary(
                 formatter,
                 &workspace_command,
-                parent_tree.diff_stream(&tree, &EverythingMatcher),
+                parent_tree.diff_stream(&tree, matcher.as_ref()),
             )?;
         }
 
+        // TODO: Conflicts should also be filtered by the `matcher`. See the related
+        // TODO on `MergedTree::conflicts()`.
         let conflicts = wc_commit.tree()?.conflicts().collect_vec();
         if !conflicts.is_empty() {
             writeln!(
                 formatter.labeled("conflict"),
                 "There are unresolved conflicts at these paths:"
             )?;
-            resolve::print_conflicted_paths(&conflicts, formatter, &workspace_command)?
+            print_conflicted_paths(&conflicts, formatter, &workspace_command)?
         }
 
         let template = workspace_command.commit_summary_template();
@@ -84,6 +92,16 @@ pub(crate) fn cmd_status(
             template.format(&parent, formatter)?;
             writeln!(formatter)?;
         }
+
+        let wc_revset = RevsetExpression::commit(wc_commit.id().clone());
+        // Ancestors with conflicts, excluding the current working copy commit.
+        let ancestors_conflicts = RevsetExpression::filter(RevsetFilterPredicate::HasConflict)
+            .intersection(&wc_revset.ancestors())
+            .minus(&wc_revset)
+            .evaluate_programmatic(repo.as_ref())?
+            .iter()
+            .collect();
+        workspace_command.report_repo_conflicts(formatter, repo, ancestors_conflicts)?;
     } else {
         writeln!(formatter, "No working copy")?;
     }
