@@ -149,6 +149,48 @@ fn test_diff_basic() {
     file3 | 1 +
     3 files changed, 2 insertions(+), 1 deletion(-)
     "###);
+
+    // Filter by glob pattern
+    let stdout = test_env.jj_cmd_success(
+        &repo_path,
+        &[
+            "diff",
+            "--config-toml=ui.allow-filesets=true",
+            "-s",
+            r#"glob:"file[12]""#,
+        ],
+    );
+    insta::assert_snapshot!(stdout, @r###"
+    D file1
+    M file2
+    "###);
+
+    // Unmatched paths should generate warnings
+    let (stdout, stderr) = test_env.jj_cmd_ok(
+        test_env.env_root(),
+        &[
+            "diff",
+            "-Rrepo",
+            "-s",
+            "repo",       // matches directory
+            "repo/file1", // deleted in to_tree, but exists in from_tree
+            "repo/x",
+            "repo/y/z",
+        ],
+    );
+    insta::assert_snapshot!(stdout.replace('\\', "/"), @r###"
+    D repo/file1
+    M repo/file2
+    A repo/file3
+    "###);
+    insta::assert_snapshot!(stderr.replace('\\', "/"), @r###"
+    Warning: No matching entries for paths: repo/x, repo/y/z
+    "###);
+
+    // Unmodified paths shouldn't generate warnings
+    let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["diff", "-s", "--from=@", "file2"]);
+    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stderr, @"");
 }
 
 #[test]
@@ -199,7 +241,7 @@ fn test_diff_types() {
     std::fs::write(&file_path, "foo").unwrap();
     test_env.jj_cmd_ok(&repo_path, &["new", "root()"]);
     std::fs::write(&file_path, "bar").unwrap();
-    test_env.jj_cmd_ok(&repo_path, &["move", r#"--to=description("conflict")"#]);
+    test_env.jj_cmd_ok(&repo_path, &["squash", r#"--into=description("conflict")"#]);
 
     #[cfg(unix)]
     {
