@@ -241,15 +241,12 @@ fn test_squash_partial() {
     b
     "###);
 
-    // If we specify only a non-existent file, then the squash still succeeds and
-    // creates unchanged commits.
+    // If we specify only a non-existent file, then nothing changes.
     test_env.jj_cmd_ok(&repo_path, &["undo"]);
     let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["squash", "-r", "b", "nonexistent"]);
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
-    Rebased 2 descendant commits
-    Working copy now at: mzvwutvl 5e297967 c | (no description set)
-    Parent commit      : kkmpptxz ac258609 b | (no description set)
+    Nothing changed.
     "###);
 
     // We get a warning if we pass a positional argument that looks like a revset
@@ -257,9 +254,7 @@ fn test_squash_partial() {
     let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["squash", "b"]);
     insta::assert_snapshot!(stderr, @r###"
     Warning: The argument "b" is being interpreted as a path. To specify a revset, pass -r "b" instead.
-    Rebased 1 descendant commits
-    Working copy now at: mzvwutvl 1c4e5596 c | (no description set)
-    Parent commit      : kkmpptxz 16cc94b4 b | (no description set)
+    Nothing changed.
     "###);
     insta::assert_snapshot!(stdout, @"");
 }
@@ -570,15 +565,13 @@ fn test_squash_from_to_partial() {
     a
     "###);
 
-    // If we specify only a non-existent file, then the move still succeeds and
-    // creates unchanged commits.
+    // If we specify only a non-existent file, then nothing changes.
     test_env.jj_cmd_ok(&repo_path, &["undo"]);
     let (stdout, stderr) =
         test_env.jj_cmd_ok(&repo_path, &["squash", "--from", "c", "nonexistent"]);
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
-    Working copy now at: vruxwmqv b670567d d | (no description set)
-    Parent commit      : qpvuntsm 3db0a2f5 a | (no description set)
+    Nothing changed.
     "###);
 }
 
@@ -629,25 +622,26 @@ fn test_squash_from_multiple() {
     "###);
 
     // Squash a few commits sideways
-    let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["squash", "--from=b|c", "--into=d"]);
+    let (stdout, stderr) =
+        test_env.jj_cmd_ok(&repo_path, &["squash", "--from=b", "--from=c", "--into=d"]);
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
     Rebased 2 descendant commits
     New conflicts appeared in these commits:
-      yqosqzyt d5401742 d | (conflict) (no description set)
+      yqosqzyt 50bd7d24 d | (conflict) (no description set)
     To resolve the conflicts, start by updating to it:
       jj new yqosqzytrlsw
     Then use `jj resolve`, or edit the conflict markers in the file directly.
     Once the conflicts are resolved, you may want inspect the result with `jj diff`.
     Then run `jj squash` to move the resolution into the conflicted commit.
-    Working copy now at: kpqxywon cc9f4cad f | (no description set)
-    Parent commit      : yostqsxw 9f25b62d e | (no description set)
+    Working copy now at: kpqxywon dd653e49 f | (no description set)
+    Parent commit      : yostqsxw e40f2544 e | (no description set)
     "###);
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
-    @  cc9f4cad1a29 f
-    ◉    9f25b62ddffc e
+    @  dd653e494199 f
+    ◉    e40f2544ad31 e
     ├─╮
-    ◉ │  d54017421f3f d
+    ◉ │  50bd7d246d8e d
     ├─╯
     ◉  37941ee54ace a b c
     ◉  000000000000
@@ -690,6 +684,13 @@ fn test_squash_from_multiple() {
     insta::assert_snapshot!(stdout, @r###"
     f
     "###);
+
+    // Empty squash shouldn't crash
+    let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["squash", "--from=none()"]);
+    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Nothing changed.
+    "###);
 }
 
 #[test]
@@ -752,23 +753,23 @@ fn test_squash_from_multiple_partial() {
     insta::assert_snapshot!(stderr, @r###"
     Rebased 2 descendant commits
     New conflicts appeared in these commits:
-      yqosqzyt 13468b54 d | (conflict) (no description set)
+      yqosqzyt 85d3ae29 d | (conflict) (no description set)
     To resolve the conflicts, start by updating to it:
       jj new yqosqzytrlsw
     Then use `jj resolve`, or edit the conflict markers in the file directly.
     Once the conflicts are resolved, you may want inspect the result with `jj diff`.
     Then run `jj squash` to move the resolution into the conflicted commit.
-    Working copy now at: kpqxywon 8aaa7910 f | (no description set)
-    Parent commit      : yostqsxw 5aad25ea e | (no description set)
+    Working copy now at: kpqxywon 97861bbf f | (no description set)
+    Parent commit      : yostqsxw 2dbaf4e8 e | (no description set)
     "###);
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
-    @  8aaa79109163 f
-    ◉      5aad25eae5aa e
+    @  97861bbf7ae5 f
+    ◉      2dbaf4e8c7f7 e
     ├─┬─╮
     │ │ ◉  ba60ddff2d41 b
     │ ◉ │  8ef5a315bf7d c
     │ ├─╯
-    ◉ │  13468b546ba3 d
+    ◉ │  85d3ae290b9b d
     ├─╯
     ◉  47a1e795d146 a
     ◉  000000000000
@@ -850,8 +851,97 @@ fn test_squash_from_multiple_partial() {
     "###);
 }
 
+#[test]
+fn test_squash_from_multiple_partial_no_op() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    let repo_path = test_env.env_root().join("repo");
+
+    // Create history like this:
+    // B C D
+    //  \|/
+    //   A
+    let file_a = repo_path.join("a");
+    let file_b = repo_path.join("b");
+    let file_c = repo_path.join("c");
+    let file_d = repo_path.join("d");
+    test_env.jj_cmd_ok(&repo_path, &["describe", "-m=a"]);
+    std::fs::write(file_a, "a\n").unwrap();
+    test_env.jj_cmd_ok(&repo_path, &["new", "-m=b"]);
+    std::fs::write(file_b, "b\n").unwrap();
+    test_env.jj_cmd_ok(&repo_path, &["new", "@-", "-m=c"]);
+    std::fs::write(file_c, "c\n").unwrap();
+    test_env.jj_cmd_ok(&repo_path, &["new", "@-", "-m=d"]);
+    std::fs::write(file_d, "d\n").unwrap();
+    // Test the setup
+    insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
+    @  09441f0a6266 d
+    │ ◉  5ad3ca4090a7 c
+    ├─╯
+    │ ◉  285201979c90 b
+    ├─╯
+    ◉  3df52ee1f8a9 a
+    ◉  000000000000
+    "###);
+
+    // Source commits that didn't match the paths are not rewritten
+    let (stdout, stderr) = test_env.jj_cmd_ok(
+        &repo_path,
+        &["squash", "--from=@-+ ~ @", "--into=@", "-m=d", "b"],
+    );
+    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Working copy now at: mzvwutvl 9227d0d7 d
+    Parent commit      : qpvuntsm 3df52ee1 a
+    Added 1 files, modified 0 files, removed 0 files
+    "###);
+    insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
+    @  9227d0d780fa d
+    │ ◉  5ad3ca4090a7 c
+    ├─╯
+    ◉  3df52ee1f8a9 a
+    ◉  000000000000
+    "###);
+    let stdout = test_env.jj_cmd_success(
+        &repo_path,
+        &[
+            "obslog",
+            "-T",
+            r#"separate(" ", commit_id.short(), description)"#,
+        ],
+    );
+    insta::assert_snapshot!(stdout, @r###"
+    @    9227d0d780fa d
+    ├─╮
+    ◉ │  09441f0a6266 d
+    ◉ │  cba0f0aa472b d
+      ◉  285201979c90 b
+      ◉  81187418277d b
+    "###);
+
+    // If no source commits match the paths, then the whole operation is a no-op
+    test_env.jj_cmd_ok(&repo_path, &["undo"]);
+    let (stdout, stderr) = test_env.jj_cmd_ok(
+        &repo_path,
+        &["squash", "--from=@-+ ~ @", "--into=@", "-m=d", "a"],
+    );
+    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Nothing changed.
+    "###);
+    insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
+    @  09441f0a6266 d
+    │ ◉  5ad3ca4090a7 c
+    ├─╯
+    │ ◉  285201979c90 b
+    ├─╯
+    ◉  3df52ee1f8a9 a
+    ◉  000000000000
+    "###);
+}
+
 fn get_log_output(test_env: &TestEnvironment, repo_path: &Path) -> String {
-    let template = r#"commit_id.short() ++ " " ++ branches"#;
+    let template = r#"separate(" ", commit_id.short(), branches, description)"#;
     test_env.jj_cmd_success(repo_path, &["log", "-T", template])
 }
 
@@ -968,9 +1058,84 @@ fn test_squash_empty() {
     "###);
 }
 
+#[test]
+fn test_squash_use_destination_message() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    let repo_path = test_env.env_root().join("repo");
+
+    test_env.jj_cmd_ok(&repo_path, &["commit", "-m=a"]);
+    test_env.jj_cmd_ok(&repo_path, &["commit", "-m=b"]);
+    test_env.jj_cmd_ok(&repo_path, &["describe", "-m=c"]);
+    // Test the setup
+    insta::assert_snapshot!(get_log_output_with_description(&test_env, &repo_path), @r###"
+    @  71f7c810d8ed c
+    ◉  10dd87c3b4e2 b
+    ◉  4c5b3042d9e0 a
+    ◉  000000000000
+    "###);
+
+    // Squash the current revision using the short name for the option.
+    test_env.jj_cmd_ok(&repo_path, &["squash", "-u"]);
+    insta::assert_snapshot!(get_log_output_with_description(&test_env, &repo_path), @r###"
+    @  10e30ce4a910
+    ◉  1c21278b775f b
+    ◉  4c5b3042d9e0 a
+    ◉  000000000000
+    "###);
+
+    // Undo and squash again, but this time squash both "b" and "c" into "a".
+    test_env.jj_cmd_ok(&repo_path, &["undo"]);
+    test_env.jj_cmd_ok(
+        &repo_path,
+        &[
+            "squash",
+            "--use-destination-message",
+            "--from",
+            "description(b)::",
+            "--into",
+            "description(a)",
+        ],
+    );
+    insta::assert_snapshot!(get_log_output_with_description(&test_env, &repo_path), @r###"
+    @  da1507508bdf
+    ◉  f1387f804776 a
+    ◉  000000000000
+    "###);
+}
+
+// The --use-destination-message and --message options are incompatible.
+#[test]
+fn test_squash_use_destination_message_and_message_mutual_exclusion() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    let repo_path = test_env.env_root().join("repo");
+    test_env.jj_cmd_ok(&repo_path, &["commit", "-m=a"]);
+    test_env.jj_cmd_ok(&repo_path, &["describe", "-m=b"]);
+    insta::assert_snapshot!(test_env.jj_cmd_cli_error(
+        &repo_path,
+        &[
+            "squash",
+            "--message=123",
+            "--use-destination-message",
+        ],
+    ), @r###"
+    error: the argument '--message <MESSAGE>' cannot be used with '--use-destination-message'
+
+    Usage: jj squash --message <MESSAGE> [PATHS]...
+
+    For more information, try '--help'.
+    "###);
+}
+
 fn get_description(test_env: &TestEnvironment, repo_path: &Path, rev: &str) -> String {
     test_env.jj_cmd_success(
         repo_path,
         &["log", "--no-graph", "-T", "description", "-r", rev],
     )
 }
+
+fn get_log_output_with_description(test_env: &TestEnvironment, repo_path: &Path) -> String {
+    let template = r#"separate(" ", commit_id.short(), description)"#;
+    test_env.jj_cmd_success(repo_path, &["log", "-T", template])
+}
