@@ -29,7 +29,9 @@ use itertools::Itertools;
 use smallvec::{smallvec, SmallVec};
 use tempfile::NamedTempFile;
 
-use super::composite::{AsCompositeIndex, ChangeIdIndexImpl, CompositeIndex, IndexSegment};
+use super::composite::{
+    AsCompositeIndex, ChangeIdIndexImpl, CompositeIndex, DynIndexSegment, IndexSegment,
+};
 use super::entry::{IndexPosition, LocalPosition, SmallIndexPositionsVec, SmallLocalPositionsVec};
 use super::readonly::{
     DefaultReadonlyIndex, ReadonlyIndexSegment, INDEX_SEGMENT_FILE_FORMAT_VERSION, OVERFLOW_FLAG,
@@ -88,7 +90,7 @@ impl MutableIndexSegment {
         }
     }
 
-    pub(super) fn as_composite(&self) -> CompositeIndex {
+    pub(super) fn as_composite(&self) -> &CompositeIndex {
         CompositeIndex::new(self)
     }
 
@@ -137,7 +139,7 @@ impl MutableIndexSegment {
         self.graph.push(entry);
     }
 
-    pub(super) fn add_commits_from(&mut self, other_segment: &dyn IndexSegment) {
+    pub(super) fn add_commits_from(&mut self, other_segment: &DynIndexSegment) {
         let other = CompositeIndex::new(other_segment);
         for pos in other_segment.num_parent_commits()..other.num_commits() {
             let entry = other.entry_by_pos(IndexPosition(pos));
@@ -465,7 +467,7 @@ impl DefaultMutableIndex {
 }
 
 impl AsCompositeIndex for DefaultMutableIndex {
-    fn as_composite(&self) -> CompositeIndex<'_> {
+    fn as_composite(&self) -> &CompositeIndex {
         self.0.as_composite()
     }
 }
@@ -502,10 +504,6 @@ impl Index for DefaultMutableIndex {
         self.as_composite().heads(candidates)
     }
 
-    fn topo_order(&self, input: &mut dyn Iterator<Item = &CommitId>) -> Vec<CommitId> {
-        self.as_composite().topo_order(input)
-    }
-
     fn evaluate_revset<'index>(
         &'index self,
         expression: &ResolvedExpression,
