@@ -16,63 +16,236 @@
 
 use std::cmp::{max, Reverse};
 use std::collections::{BinaryHeap, HashMap, HashSet};
-use std::iter::FusedIterator;
+use std::iter::{Fuse, FusedIterator};
 use std::ops::Range;
 
 use smallvec::SmallVec;
 
 use super::composite::CompositeIndex;
-use super::entry::{IndexEntry, IndexPosition, SmallIndexPositionsVec};
+use super::entry::{IndexPosition, SmallIndexPositionsVec};
 
-trait RevWalkIndex<'a> {
+/// Like `Iterator`, but doesn't borrow the `index` internally.
+pub(super) trait RevWalk<I: ?Sized> {
+    type Item;
+
+    /// Advances the iteration and returns the next item.
+    ///
+    /// The caller must provide the same `index` instance.
+    ///
+    /// Returns `None` when the iteration is finished. Once `None` is returned,
+    /// this will never resume. In other words, a `RevWalk` is fused.
+    fn next(&mut self, index: &I) -> Option<Self::Item>;
+
+    // The following methods are provided for convenience. They are not supposed
+    // to be reimplemented.
+
+    /// Wraps in adapter that will filter items by the given `predicate`.
+    fn filter<P>(self, predicate: P) -> FilterRevWalk<Self, P>
+    where
+        Self: Sized,
+        P: FnMut(&I, &Self::Item) -> bool,
+    {
+        FilterRevWalk {
+            walk: self,
+            predicate,
+        }
+    }
+
+    /// Wraps in adapter that can peek one more item without consuming.
+    fn peekable(self) -> PeekableRevWalk<I, Self>
+    where
+        Self: Sized,
+    {
+        PeekableRevWalk {
+            walk: self,
+            peeked: None,
+        }
+    }
+
+    /// Reattaches the underlying `index`.
+    fn attach(self, index: &I) -> RevWalkBorrowedIndexIter<'_, I, Self>
+    where
+        Self: Sized,
+    {
+        RevWalkBorrowedIndexIter { index, walk: self }
+    }
+}
+
+impl<I: ?Sized, W: RevWalk<I> + ?Sized> RevWalk<I> for Box<W> {
+    type Item = W::Item;
+
+    fn next(&mut self, index: &I) -> Option<Self::Item> {
+        <W as RevWalk<I>>::next(self, index)
+    }
+}
+
+/// Adapter that turns `Iterator` into `RevWalk` by dropping index argument.
+///
+/// As the name suggests, the source object is usually a slice or `Vec`.
+#[derive(Clone, Debug)]
+pub(super) struct EagerRevWalk<T> {
+    iter: Fuse<T>,
+}
+
+impl<T: Iterator> EagerRevWalk<T> {
+    pub fn new(iter: T) -> Self {
+        EagerRevWalk { iter: iter.fuse() }
+    }
+}
+
+impl<I: ?Sized, T: Iterator> RevWalk<I> for EagerRevWalk<T> {
+    type Item = T::Item;
+
+    fn next(&mut self, _index: &I) -> Option<Self::Item> {
+        self.iter.next()
+    }
+}
+
+#[derive(Clone, Debug)]
+#[must_use]
+pub(super) struct FilterRevWalk<W, P> {
+    walk: W,
+    predicate: P,
+}
+
+impl<I, W, P> RevWalk<I> for FilterRevWalk<W, P>
+where
+    I: ?Sized,
+    W: RevWalk<I>,
+    P: FnMut(&I, &W::Item) -> bool,
+{
+    type Item = W::Item;
+
+    fn next(&mut self, index: &I) -> Option<Self::Item> {
+        while let Some(item) = self.walk.next(index) {
+            if (self.predicate)(index, &item) {
+                return Some(item);
+            }
+        }
+        None
+    }
+}
+
+#[derive(Clone, Debug)]
+#[must_use]
+pub(super) struct PeekableRevWalk<I: ?Sized, W: RevWalk<I>> {
+    walk: W,
+    // Since RevWalk is fused, we don't need a nested Option<Option<_>>.
+    peeked: Option<W::Item>,
+}
+
+impl<I: ?Sized, W: RevWalk<I>> PeekableRevWalk<I, W> {
+    pub fn peek(&mut self, index: &I) -> Option<&W::Item> {
+        if self.peeked.is_none() {
+            self.peeked = self.walk.next(index);
+        }
+        self.peeked.as_ref()
+    }
+
+    pub fn next_if(
+        &mut self,
+        index: &I,
+        predicate: impl FnOnce(&W::Item) -> bool,
+    ) -> Option<W::Item> {
+        match self.next(index) {
+            Some(item) if predicate(&item) => Some(item),
+            other => {
+                assert!(self.peeked.is_none());
+                self.peeked = other;
+                None
+            }
+        }
+    }
+}
+
+impl<I: ?Sized, W: RevWalk<I>> RevWalk<I> for PeekableRevWalk<I, W> {
+    type Item = W::Item;
+
+    fn next(&mut self, index: &I) -> Option<Self::Item> {
+        self.peeked.take().or_else(|| self.walk.next(index))
+    }
+}
+
+/// Adapter that turns `RevWalk` into `Iterator` by attaching borrowed `index`.
+#[derive(Clone, Debug)]
+#[must_use]
+pub(super) struct RevWalkBorrowedIndexIter<'a, I: ?Sized, W> {
+    index: &'a I,
+    walk: W,
+}
+
+impl<I: ?Sized, W> RevWalkBorrowedIndexIter<'_, I, W> {
+    /// Turns into `'static`-lifetime walk object by detaching the index.
+    pub fn detach(self) -> W {
+        self.walk
+    }
+}
+
+impl<I: ?Sized, W: RevWalk<I>> Iterator for RevWalkBorrowedIndexIter<'_, I, W> {
+    type Item = W::Item;
+
+    fn next(&mut self) -> Option<Self::Item> {
+        self.walk.next(self.index)
+    }
+}
+
+impl<I: ?Sized, W: RevWalk<I>> FusedIterator for RevWalkBorrowedIndexIter<'_, I, W> {}
+
+/// Adapter that turns `RevWalk` into `Iterator` by attaching owned `index`.
+#[derive(Clone, Debug)]
+#[must_use]
+pub(super) struct RevWalkOwnedIndexIter<I, W> {
+    index: I,
+    walk: W,
+}
+
+impl<I, W: RevWalk<I>> Iterator for RevWalkOwnedIndexIter<I, W> {
+    type Item = W::Item;
+
+    fn next(&mut self) -> Option<Self::Item> {
+        self.walk.next(&self.index)
+    }
+}
+
+impl<I, W: RevWalk<I>> FusedIterator for RevWalkOwnedIndexIter<I, W> {}
+
+pub(super) trait RevWalkIndex {
     type Position: Copy + Ord;
     type AdjacentPositions: IntoIterator<Item = Self::Position>;
 
-    fn entry_by_pos(&self, pos: Self::Position) -> IndexEntry<'a>;
-    fn adjacent_positions(&self, entry: &IndexEntry<'_>) -> Self::AdjacentPositions;
+    fn adjacent_positions(&self, pos: Self::Position) -> Self::AdjacentPositions;
 }
 
-impl<'a> RevWalkIndex<'a> for CompositeIndex<'a> {
+impl RevWalkIndex for CompositeIndex {
     type Position = IndexPosition;
     type AdjacentPositions = SmallIndexPositionsVec;
 
-    fn entry_by_pos(&self, pos: Self::Position) -> IndexEntry<'a> {
-        CompositeIndex::entry_by_pos(self, pos)
-    }
-
-    fn adjacent_positions(&self, entry: &IndexEntry<'_>) -> Self::AdjacentPositions {
-        entry.parent_positions()
+    fn adjacent_positions(&self, pos: Self::Position) -> Self::AdjacentPositions {
+        self.entry_by_pos(pos).parent_positions()
     }
 }
 
 #[derive(Clone)]
-struct RevWalkDescendantsIndex<'a> {
-    index: CompositeIndex<'a>,
+pub(super) struct RevWalkDescendantsIndex {
     children_map: HashMap<IndexPosition, DescendantIndexPositionsVec>,
 }
 
 // See SmallIndexPositionsVec for the array size.
 type DescendantIndexPositionsVec = SmallVec<[Reverse<IndexPosition>; 4]>;
 
-impl<'a> RevWalkDescendantsIndex<'a> {
-    fn build<'b>(
-        index: CompositeIndex<'a>,
-        entries: impl IntoIterator<Item = IndexEntry<'b>>,
-    ) -> Self {
+impl RevWalkDescendantsIndex {
+    fn build(index: &CompositeIndex, positions: impl IntoIterator<Item = IndexPosition>) -> Self {
         // For dense set, it's probably cheaper to use `Vec` instead of `HashMap`.
         let mut children_map: HashMap<IndexPosition, DescendantIndexPositionsVec> = HashMap::new();
-        for entry in entries {
-            children_map.entry(entry.position()).or_default(); // mark head node
-            for parent_pos in entry.parent_positions() {
+        for pos in positions {
+            children_map.entry(pos).or_default(); // mark head node
+            for parent_pos in index.entry_by_pos(pos).parent_positions() {
                 let parent = children_map.entry(parent_pos).or_default();
-                parent.push(Reverse(entry.position()));
+                parent.push(Reverse(pos));
             }
         }
 
-        RevWalkDescendantsIndex {
-            index,
-            children_map,
-        }
+        RevWalkDescendantsIndex { children_map }
     }
 
     fn contains_pos(&self, pos: IndexPosition) -> bool {
@@ -80,16 +253,12 @@ impl<'a> RevWalkDescendantsIndex<'a> {
     }
 }
 
-impl<'a> RevWalkIndex<'a> for RevWalkDescendantsIndex<'a> {
+impl RevWalkIndex for RevWalkDescendantsIndex {
     type Position = Reverse<IndexPosition>;
     type AdjacentPositions = DescendantIndexPositionsVec;
 
-    fn entry_by_pos(&self, pos: Self::Position) -> IndexEntry<'a> {
-        self.index.entry_by_pos(pos.0)
-    }
-
-    fn adjacent_positions(&self, entry: &IndexEntry<'_>) -> Self::AdjacentPositions {
-        self.children_map[&entry.position()].clone()
+    fn adjacent_positions(&self, pos: Self::Position) -> Self::AdjacentPositions {
+        self.children_map[&pos.0].clone()
     }
 }
 
@@ -110,49 +279,36 @@ impl<P, T> RevWalkWorkItem<P, T> {
     fn is_wanted(&self) -> bool {
         matches!(self.state, RevWalkWorkItemState::Wanted(_))
     }
-
-    fn map_wanted<U>(self, f: impl FnOnce(T) -> U) -> RevWalkWorkItem<P, U> {
-        RevWalkWorkItem {
-            pos: self.pos,
-            state: match self.state {
-                RevWalkWorkItemState::Wanted(t) => RevWalkWorkItemState::Wanted(f(t)),
-                RevWalkWorkItemState::Unwanted => RevWalkWorkItemState::Unwanted,
-            },
-        }
-    }
 }
 
 #[derive(Clone)]
 struct RevWalkQueue<P, T> {
     items: BinaryHeap<RevWalkWorkItem<P, T>>,
+    min_pos: P,
     unwanted_count: usize,
 }
 
 impl<P: Ord, T: Ord> RevWalkQueue<P, T> {
-    fn new() -> Self {
+    fn with_min_pos(min_pos: P) -> Self {
         Self {
             items: BinaryHeap::new(),
+            min_pos,
             unwanted_count: 0,
         }
     }
 
-    fn map_wanted<U: Ord>(self, mut f: impl FnMut(T) -> U) -> RevWalkQueue<P, U> {
-        RevWalkQueue {
-            items: self
-                .items
-                .into_iter()
-                .map(|x| x.map_wanted(&mut f))
-                .collect(),
-            unwanted_count: self.unwanted_count,
-        }
-    }
-
     fn push_wanted(&mut self, pos: P, t: T) {
+        if pos < self.min_pos {
+            return;
+        }
         let state = RevWalkWorkItemState::Wanted(t);
         self.items.push(RevWalkWorkItem { pos, state });
     }
 
     fn push_unwanted(&mut self, pos: P) {
+        if pos < self.min_pos {
+            return;
+        }
         let state = RevWalkWorkItemState::Unwanted;
         self.items.push(RevWalkWorkItem { pos, state });
         self.unwanted_count += 1;
@@ -200,31 +356,69 @@ impl<P: Ord, T: Ord> RevWalkQueue<P, T> {
 }
 
 #[derive(Clone)]
-pub struct RevWalk<'a>(RevWalkImpl<'a, CompositeIndex<'a>>);
+#[must_use]
+pub(super) struct RevWalkBuilder<'a> {
+    index: &'a CompositeIndex,
+    wanted: Vec<IndexPosition>,
+    unwanted: Vec<IndexPosition>,
+}
 
-impl<'a> RevWalk<'a> {
-    pub(super) fn new(index: CompositeIndex<'a>) -> Self {
-        let queue = RevWalkQueue::new();
-        RevWalk(RevWalkImpl { index, queue })
+impl<'a> RevWalkBuilder<'a> {
+    pub fn new(index: &'a CompositeIndex) -> Self {
+        RevWalkBuilder {
+            index,
+            wanted: Vec::new(),
+            unwanted: Vec::new(),
+        }
     }
 
-    pub(super) fn extend_wanted(&mut self, positions: impl IntoIterator<Item = IndexPosition>) {
-        self.0.queue.extend_wanted(positions, ());
+    /// Adds head positions to be included.
+    pub fn wanted_heads(mut self, positions: impl IntoIterator<Item = IndexPosition>) -> Self {
+        self.wanted.extend(positions);
+        self
     }
 
-    pub(super) fn extend_unwanted(&mut self, positions: impl IntoIterator<Item = IndexPosition>) {
-        self.0.queue.extend_unwanted(positions);
+    /// Adds root positions to be excluded. The roots precede the heads.
+    pub fn unwanted_roots(mut self, positions: impl IntoIterator<Item = IndexPosition>) -> Self {
+        self.unwanted.extend(positions);
+        self
     }
 
-    /// Filters entries by generation (or depth from the current wanted set.)
+    /// Walks ancestors.
+    pub fn ancestors(self) -> RevWalkAncestors<'a> {
+        self.ancestors_with_min_pos(IndexPosition::MIN)
+    }
+
+    fn ancestors_with_min_pos(self, min_pos: IndexPosition) -> RevWalkAncestors<'a> {
+        let index = self.index;
+        let mut queue = RevWalkQueue::with_min_pos(min_pos);
+        queue.extend_wanted(self.wanted, ());
+        queue.extend_unwanted(self.unwanted);
+        RevWalkBorrowedIndexIter {
+            index,
+            walk: RevWalkImpl { queue },
+        }
+    }
+
+    /// Walks ancestors within the `generation_range`.
     ///
-    /// The generation of the current wanted entries starts from 0.
-    pub fn filter_by_generation(self, generation_range: Range<u32>) -> RevWalkGenerationRange<'a> {
-        RevWalkGenerationRange(RevWalkGenerationRangeImpl::new(
-            self.0.index,
-            self.0.queue,
-            generation_range,
-        ))
+    /// A generation number counts from the heads.
+    pub fn ancestors_filtered_by_generation(
+        self,
+        generation_range: Range<u32>,
+    ) -> RevWalkAncestorsGenerationRange<'a> {
+        let index = self.index;
+        let mut queue = RevWalkQueue::with_min_pos(IndexPosition::MIN);
+        let item_range = RevWalkItemGenerationRange::from_filter_range(generation_range.clone());
+        queue.extend_wanted(self.wanted, Reverse(item_range));
+        queue.extend_unwanted(self.unwanted);
+        RevWalkBorrowedIndexIter {
+            index,
+            walk: RevWalkGenerationRangeImpl {
+                queue,
+                generation_end: generation_range.end,
+            },
+        }
     }
 
     /// Walks ancestors until all of the reachable roots in `root_positions` get
@@ -232,88 +426,104 @@ impl<'a> RevWalk<'a> {
     ///
     /// Use this if you are only interested in descendants of the given roots.
     /// The caller still needs to filter out unwanted entries.
-    pub fn take_until_roots(
+    pub fn ancestors_until_roots(
         self,
-        root_positions: &[IndexPosition],
-    ) -> impl Iterator<Item = IndexEntry<'a>> + Clone + 'a {
+        root_positions: impl IntoIterator<Item = IndexPosition>,
+    ) -> RevWalkAncestors<'a> {
         // We can also make it stop visiting based on the generation number. Maybe
         // it will perform better for unbalanced branchy history.
         // https://github.com/martinvonz/jj/pull/1492#discussion_r1160678325
-        let bottom_position = *root_positions.iter().min().unwrap_or(&IndexPosition::MAX);
-        self.take_while(move |entry| entry.position() >= bottom_position)
+        let min_pos = root_positions
+            .into_iter()
+            .min()
+            .unwrap_or(IndexPosition::MAX);
+        self.ancestors_with_min_pos(min_pos)
     }
 
-    /// Fully consumes the ancestors and walks back from `root_positions`.
+    /// Fully consumes ancestors and walks back from the `root_positions`.
     ///
     /// The returned iterator yields entries in order of ascending index
     /// position.
-    pub fn descendants(self, root_positions: &[IndexPosition]) -> RevWalkDescendants<'a> {
-        RevWalkDescendants {
-            candidate_entries: self.take_until_roots(root_positions).collect(),
-            root_positions: root_positions.iter().copied().collect(),
-            reachable_positions: HashSet::new(),
+    pub fn descendants(
+        self,
+        root_positions: impl IntoIterator<Item = IndexPosition>,
+    ) -> RevWalkDescendants<'a> {
+        let index = self.index;
+        let root_positions = HashSet::from_iter(root_positions);
+        let candidate_positions = self
+            .ancestors_until_roots(root_positions.iter().copied())
+            .collect();
+        RevWalkBorrowedIndexIter {
+            index,
+            walk: RevWalkDescendantsImpl {
+                candidate_positions,
+                root_positions,
+                reachable_positions: HashSet::new(),
+            },
         }
     }
 
-    /// Fully consumes the ancestors and walks back from `root_positions` within
-    /// `generation_range`.
+    /// Fully consumes ancestors and walks back from the `root_positions` within
+    /// the `generation_range`.
+    ///
+    /// A generation number counts from the roots.
     ///
     /// The returned iterator yields entries in order of ascending index
     /// position.
     pub fn descendants_filtered_by_generation(
         self,
-        root_positions: &[IndexPosition],
+        root_positions: impl IntoIterator<Item = IndexPosition>,
         generation_range: Range<u32>,
-    ) -> RevWalkDescendantsGenerationRange<'a> {
-        let index = self.0.index;
-        let entries = self.take_until_roots(root_positions);
-        let descendants_index = RevWalkDescendantsIndex::build(index, entries);
-        let mut queue = RevWalkQueue::new();
-        for &pos in root_positions {
+    ) -> RevWalkDescendantsGenerationRange {
+        let index = self.index;
+        let root_positions = Vec::from_iter(root_positions);
+        let positions = self.ancestors_until_roots(root_positions.iter().copied());
+        let descendants_index = RevWalkDescendantsIndex::build(index, positions);
+
+        let mut queue = RevWalkQueue::with_min_pos(Reverse(IndexPosition::MAX));
+        let item_range = RevWalkItemGenerationRange::from_filter_range(generation_range.clone());
+        for pos in root_positions {
             // Do not add unreachable roots which shouldn't be visited
             if descendants_index.contains_pos(pos) {
-                queue.push_wanted(Reverse(pos), ());
+                queue.push_wanted(Reverse(pos), Reverse(item_range));
             }
         }
-        RevWalkDescendantsGenerationRange(RevWalkGenerationRangeImpl::new(
-            descendants_index,
-            queue,
-            generation_range,
-        ))
+        RevWalkOwnedIndexIter {
+            index: descendants_index,
+            walk: RevWalkGenerationRangeImpl {
+                queue,
+                generation_end: generation_range.end,
+            },
+        }
     }
 }
 
-impl<'a> Iterator for RevWalk<'a> {
-    type Item = IndexEntry<'a>;
-
-    fn next(&mut self) -> Option<Self::Item> {
-        self.0.next()
-    }
-}
+pub(super) type RevWalkAncestors<'a> =
+    RevWalkBorrowedIndexIter<'a, CompositeIndex, RevWalkImpl<IndexPosition>>;
 
 #[derive(Clone)]
-struct RevWalkImpl<'a, I: RevWalkIndex<'a>> {
-    index: I,
-    queue: RevWalkQueue<I::Position, ()>,
+#[must_use]
+pub(super) struct RevWalkImpl<P> {
+    queue: RevWalkQueue<P, ()>,
 }
 
-impl<'a, I: RevWalkIndex<'a>> RevWalkImpl<'a, I> {
-    fn next(&mut self) -> Option<IndexEntry<'a>> {
+impl<I: RevWalkIndex + ?Sized> RevWalk<I> for RevWalkImpl<I::Position> {
+    type Item = I::Position;
+
+    fn next(&mut self, index: &I) -> Option<Self::Item> {
         while let Some(item) = self.queue.pop() {
             self.queue.skip_while_eq(&item.pos);
             if item.is_wanted() {
-                let entry = self.index.entry_by_pos(item.pos);
                 self.queue
-                    .extend_wanted(self.index.adjacent_positions(&entry), ());
-                return Some(entry);
+                    .extend_wanted(index.adjacent_positions(item.pos), ());
+                return Some(item.pos);
             } else if self.queue.items.len() == self.queue.unwanted_count {
                 // No more wanted entries to walk
                 debug_assert!(!self.queue.items.iter().any(|x| x.is_wanted()));
                 return None;
             } else {
-                let entry = self.index.entry_by_pos(item.pos);
                 self.queue
-                    .extend_unwanted(self.index.adjacent_positions(&entry));
+                    .extend_unwanted(index.adjacent_positions(item.pos));
             }
         }
 
@@ -325,64 +535,26 @@ impl<'a, I: RevWalkIndex<'a>> RevWalkImpl<'a, I> {
     }
 }
 
-#[derive(Clone)]
-pub struct RevWalkGenerationRange<'a>(RevWalkGenerationRangeImpl<'a, CompositeIndex<'a>>);
-
-impl<'a> Iterator for RevWalkGenerationRange<'a> {
-    type Item = IndexEntry<'a>;
-
-    fn next(&mut self) -> Option<Self::Item> {
-        self.0.next()
-    }
-}
+pub(super) type RevWalkAncestorsGenerationRange<'a> =
+    RevWalkBorrowedIndexIter<'a, CompositeIndex, RevWalkGenerationRangeImpl<IndexPosition>>;
+pub(super) type RevWalkDescendantsGenerationRange = RevWalkOwnedIndexIter<
+    RevWalkDescendantsIndex,
+    RevWalkGenerationRangeImpl<Reverse<IndexPosition>>,
+>;
 
 #[derive(Clone)]
-pub struct RevWalkDescendantsGenerationRange<'a>(
-    RevWalkGenerationRangeImpl<'a, RevWalkDescendantsIndex<'a>>,
-);
-
-impl<'a> Iterator for RevWalkDescendantsGenerationRange<'a> {
-    type Item = IndexEntry<'a>;
-
-    fn next(&mut self) -> Option<Self::Item> {
-        self.0.next()
-    }
-}
-
-#[derive(Clone)]
-struct RevWalkGenerationRangeImpl<'a, I: RevWalkIndex<'a>> {
-    index: I,
+#[must_use]
+pub(super) struct RevWalkGenerationRangeImpl<P> {
     // Sort item generations in ascending order
-    queue: RevWalkQueue<I::Position, Reverse<RevWalkItemGenerationRange>>,
+    queue: RevWalkQueue<P, Reverse<RevWalkItemGenerationRange>>,
     generation_end: u32,
 }
 
-impl<'a, I: RevWalkIndex<'a>> RevWalkGenerationRangeImpl<'a, I> {
-    fn new(index: I, queue: RevWalkQueue<I::Position, ()>, generation_range: Range<u32>) -> Self {
-        // Translate filter range to item ranges so that overlapped ranges can be
-        // merged later.
-        //
-        // Example: `generation_range = 1..4`
-        //     (original)                       (translated)
-        //     0 1 2 3 4                        0 1 2 3 4
-        //       *=====o  generation_range              +  generation_end
-        //     + :     :  item's generation     o=====* :  item's range
-        let item_range = RevWalkItemGenerationRange {
-            start: 0,
-            end: u32::saturating_sub(generation_range.end, generation_range.start),
-        };
-        RevWalkGenerationRangeImpl {
-            index,
-            queue: queue.map_wanted(|()| Reverse(item_range)),
-            generation_end: generation_range.end,
-        }
-    }
-
-    fn enqueue_wanted_adjacents(
-        &mut self,
-        entry: &IndexEntry<'_>,
-        gen: RevWalkItemGenerationRange,
-    ) {
+impl<P: Ord> RevWalkGenerationRangeImpl<P> {
+    fn enqueue_wanted_adjacents<I>(&mut self, index: &I, pos: P, gen: RevWalkItemGenerationRange)
+    where
+        I: RevWalkIndex<Position = P> + ?Sized,
+    {
         // `gen.start` is incremented from 0, which should never overflow
         if gen.start + 1 >= self.generation_end {
             return;
@@ -392,13 +564,16 @@ impl<'a, I: RevWalkIndex<'a>> RevWalkGenerationRangeImpl<'a, I> {
             end: gen.end.saturating_add(1),
         };
         self.queue
-            .extend_wanted(self.index.adjacent_positions(entry), Reverse(succ_gen));
+            .extend_wanted(index.adjacent_positions(pos), Reverse(succ_gen));
     }
+}
 
-    fn next(&mut self) -> Option<IndexEntry<'a>> {
+impl<I: RevWalkIndex + ?Sized> RevWalk<I> for RevWalkGenerationRangeImpl<I::Position> {
+    type Item = I::Position;
+
+    fn next(&mut self, index: &I) -> Option<Self::Item> {
         while let Some(item) = self.queue.pop() {
             if let RevWalkWorkItemState::Wanted(Reverse(mut pending_gen)) = item.state {
-                let entry = self.index.entry_by_pos(item.pos);
                 let mut some_in_range = pending_gen.contains_end(self.generation_end);
                 while let Some(x) = self.queue.pop_eq(&item.pos) {
                     // Merge overlapped ranges to reduce number of the queued items.
@@ -410,26 +585,25 @@ impl<'a, I: RevWalkIndex<'a>> RevWalkGenerationRangeImpl<'a, I> {
                         pending_gen = if let Some(merged) = pending_gen.try_merge_end(gen) {
                             merged
                         } else {
-                            self.enqueue_wanted_adjacents(&entry, pending_gen);
+                            self.enqueue_wanted_adjacents(index, item.pos, pending_gen);
                             gen
                         };
                     } else {
                         unreachable!("no more unwanted items of the same entry");
                     }
                 }
-                self.enqueue_wanted_adjacents(&entry, pending_gen);
+                self.enqueue_wanted_adjacents(index, item.pos, pending_gen);
                 if some_in_range {
-                    return Some(entry);
+                    return Some(item.pos);
                 }
             } else if self.queue.items.len() == self.queue.unwanted_count {
                 // No more wanted entries to walk
                 debug_assert!(!self.queue.items.iter().any(|x| x.is_wanted()));
                 return None;
             } else {
-                let entry = self.index.entry_by_pos(item.pos);
                 self.queue.skip_while_eq(&item.pos);
                 self.queue
-                    .extend_unwanted(self.index.adjacent_positions(&entry));
+                    .extend_unwanted(index.adjacent_positions(item.pos));
             }
         }
 
@@ -448,6 +622,23 @@ struct RevWalkItemGenerationRange {
 }
 
 impl RevWalkItemGenerationRange {
+    /// Translates filter range to item range so that overlapped ranges can be
+    /// merged later.
+    ///
+    /// Example: `generation_range = 1..4`
+    /// ```text
+    ///     (original)                       (translated)
+    ///     0 1 2 3 4                        0 1 2 3 4
+    ///       *=====o  generation_range              +  generation_end
+    ///     + :     :  item's generation     o=====* :  item's range
+    /// ```
+    fn from_filter_range(range: Range<u32>) -> Self {
+        RevWalkItemGenerationRange {
+            start: 0,
+            end: u32::saturating_sub(range.end, range.start),
+        }
+    }
+
     /// Suppose sorted ranges `self, other`, merges them if overlapped.
     #[must_use]
     fn try_merge_end(self, other: Self) -> Option<Self> {
@@ -464,9 +655,13 @@ impl RevWalkItemGenerationRange {
 }
 
 /// Walks descendants from the roots, in order of ascending index position.
+pub(super) type RevWalkDescendants<'a> =
+    RevWalkBorrowedIndexIter<'a, CompositeIndex, RevWalkDescendantsImpl>;
+
 #[derive(Clone)]
-pub struct RevWalkDescendants<'a> {
-    candidate_entries: Vec<IndexEntry<'a>>,
+#[must_use]
+pub(super) struct RevWalkDescendantsImpl {
+    candidate_positions: Vec<IndexPosition>,
     root_positions: HashSet<IndexPosition>,
     reachable_positions: HashSet<IndexPosition>,
 }
@@ -474,41 +669,39 @@ pub struct RevWalkDescendants<'a> {
 impl RevWalkDescendants<'_> {
     /// Builds a set of index positions reachable from the roots.
     ///
-    /// This is equivalent to `.map(|entry| entry.position()).collect()` on
-    /// the new iterator, but returns the internal buffer instead.
+    /// This is equivalent to `.collect()` on the new iterator, but returns the
+    /// internal buffer instead.
     pub fn collect_positions_set(mut self) -> HashSet<IndexPosition> {
         self.by_ref().for_each(drop);
-        self.reachable_positions
+        self.walk.reachable_positions
     }
 }
 
-impl<'a> Iterator for RevWalkDescendants<'a> {
-    type Item = IndexEntry<'a>;
+impl RevWalk<CompositeIndex> for RevWalkDescendantsImpl {
+    type Item = IndexPosition;
 
-    fn next(&mut self) -> Option<Self::Item> {
-        while let Some(candidate) = self.candidate_entries.pop() {
-            if self.root_positions.contains(&candidate.position())
-                || candidate
+    fn next(&mut self, index: &CompositeIndex) -> Option<Self::Item> {
+        while let Some(candidate_pos) = self.candidate_positions.pop() {
+            if self.root_positions.contains(&candidate_pos)
+                || index
+                    .entry_by_pos(candidate_pos)
                     .parent_positions()
                     .iter()
                     .any(|parent_pos| self.reachable_positions.contains(parent_pos))
             {
-                self.reachable_positions.insert(candidate.position());
-                return Some(candidate);
+                self.reachable_positions.insert(candidate_pos);
+                return Some(candidate_pos);
             }
         }
         None
     }
 }
 
-impl FusedIterator for RevWalkDescendants<'_> {}
-
 /// Computes ancestors set lazily.
 ///
 /// This is similar to `RevWalk` functionality-wise, but implemented with the
 /// different design goals:
 ///
-/// * lazy updates with no lifetimed fields
 /// * optimized for dense ancestors set
 /// * optimized for testing set membership
 /// * no iterator API (which could be implemented on top)
@@ -549,7 +742,7 @@ impl AncestorsBitSet {
     }
 
     /// Updates set by visiting ancestors until the given `to_visit_pos`.
-    pub fn visit_until(&mut self, index: CompositeIndex, to_visit_pos: IndexPosition) {
+    pub fn visit_until(&mut self, index: &CompositeIndex, to_visit_pos: IndexPosition) {
         let to_visit_bitset_pos = to_visit_pos.0 / u64::BITS;
         if to_visit_bitset_pos >= self.last_visited_bitset_pos {
             return;
@@ -596,7 +789,7 @@ mod tests {
         move || iter.next().unwrap()
     }
 
-    fn to_positions_vec(index: CompositeIndex<'_>, commit_ids: &[CommitId]) -> Vec<IndexPosition> {
+    fn to_positions_vec(index: &CompositeIndex, commit_ids: &[CommitId]) -> Vec<IndexPosition> {
         commit_ids
             .iter()
             .map(|id| index.commit_id_to_pos(id).unwrap())
@@ -604,7 +797,42 @@ mod tests {
     }
 
     #[test]
-    fn test_walk_revs() {
+    fn test_peekable_rev_walk() {
+        let source = EagerRevWalk::new(vec![0, 1, 2, 3].into_iter());
+        let mut peekable = source.peekable();
+        assert_eq!(peekable.peek(&()), Some(&0));
+        assert_eq!(peekable.peek(&()), Some(&0));
+        assert_eq!(peekable.next(&()), Some(0));
+        assert_eq!(peekable.peeked, None);
+        assert_eq!(peekable.next_if(&(), |&v| v == 2), None);
+        assert_eq!(peekable.next(&()), Some(1));
+        assert_eq!(peekable.next_if(&(), |&v| v == 2), Some(2));
+        assert_eq!(peekable.peeked, None);
+        assert_eq!(peekable.peek(&()), Some(&3));
+        assert_eq!(peekable.next_if(&(), |&v| v == 3), Some(3));
+        assert_eq!(peekable.peeked, None);
+        assert_eq!(peekable.next(&()), None);
+        assert_eq!(peekable.next(&()), None);
+
+        let source = EagerRevWalk::new((vec![] as Vec<i32>).into_iter());
+        let mut peekable = source.peekable();
+        assert_eq!(peekable.peek(&()), None);
+        assert_eq!(peekable.next(&()), None);
+    }
+
+    #[test]
+    fn test_filter_rev_walk() {
+        let source = EagerRevWalk::new(vec![0, 1, 2, 3, 4].into_iter());
+        let mut filtered = source.filter(|_, &v| v & 1 == 0);
+        assert_eq!(filtered.next(&()), Some(0));
+        assert_eq!(filtered.next(&()), Some(2));
+        assert_eq!(filtered.next(&()), Some(4));
+        assert_eq!(filtered.next(&()), None);
+        assert_eq!(filtered.next(&()), None);
+    }
+
+    #[test]
+    fn test_walk_ancestors() {
         let mut new_change_id = change_id_generator();
         let mut index = DefaultMutableIndex::full(3, 16);
         // 5
@@ -629,11 +857,11 @@ mod tests {
 
         let walk_commit_ids = |wanted: &[CommitId], unwanted: &[CommitId]| {
             let index = index.as_composite();
-            let wanted_positions = to_positions_vec(index, wanted);
-            let unwanted_positions = to_positions_vec(index, unwanted);
-            index
-                .walk_revs(&wanted_positions, &unwanted_positions)
-                .map(|entry| entry.commit_id())
+            RevWalkBuilder::new(index)
+                .wanted_heads(to_positions_vec(index, wanted))
+                .unwanted_roots(to_positions_vec(index, unwanted))
+                .ancestors()
+                .map(|pos| index.entry_by_pos(pos).commit_id())
                 .collect_vec()
         };
 
@@ -686,7 +914,64 @@ mod tests {
     }
 
     #[test]
-    fn test_walk_revs_filter_by_generation() {
+    fn test_walk_ancestors_until_roots() {
+        let mut new_change_id = change_id_generator();
+        let mut index = DefaultMutableIndex::full(3, 16);
+        //   7
+        // 6 |
+        // 5 |
+        // 4 |
+        // | 3
+        // | 2
+        // |/
+        // 1
+        // 0
+        let id_0 = CommitId::from_hex("000000");
+        let id_1 = CommitId::from_hex("111111");
+        let id_2 = CommitId::from_hex("222222");
+        let id_3 = CommitId::from_hex("333333");
+        let id_4 = CommitId::from_hex("444444");
+        let id_5 = CommitId::from_hex("555555");
+        let id_6 = CommitId::from_hex("666666");
+        let id_7 = CommitId::from_hex("777777");
+        index.add_commit_data(id_0.clone(), new_change_id(), &[]);
+        index.add_commit_data(id_1.clone(), new_change_id(), &[id_0.clone()]);
+        index.add_commit_data(id_2.clone(), new_change_id(), &[id_1.clone()]);
+        index.add_commit_data(id_3.clone(), new_change_id(), &[id_2.clone()]);
+        index.add_commit_data(id_4.clone(), new_change_id(), &[id_1.clone()]);
+        index.add_commit_data(id_5.clone(), new_change_id(), &[id_4.clone()]);
+        index.add_commit_data(id_6.clone(), new_change_id(), &[id_5.clone()]);
+        index.add_commit_data(id_7.clone(), new_change_id(), &[id_3.clone()]);
+
+        let index = index.as_composite();
+        let make_iter = |heads: &[CommitId], roots: &[CommitId]| {
+            RevWalkBuilder::new(index)
+                .wanted_heads(to_positions_vec(index, heads))
+                .ancestors_until_roots(to_positions_vec(index, roots))
+        };
+        let to_commit_id = |pos| index.entry_by_pos(pos).commit_id();
+
+        let mut iter = make_iter(&[id_6.clone(), id_7.clone()], &[id_3.clone()]);
+        assert_eq!(iter.walk.queue.items.len(), 2);
+        assert_eq!(iter.next().map(to_commit_id), Some(id_7.clone()));
+        assert_eq!(iter.next().map(to_commit_id), Some(id_6.clone()));
+        assert_eq!(iter.next().map(to_commit_id), Some(id_5.clone()));
+        assert_eq!(iter.walk.queue.items.len(), 2);
+        assert_eq!(iter.next().map(to_commit_id), Some(id_4.clone()));
+        assert_eq!(iter.walk.queue.items.len(), 1); // id_1 shouldn't be queued
+        assert_eq!(iter.next().map(to_commit_id), Some(id_3.clone()));
+        assert_eq!(iter.walk.queue.items.len(), 0); // id_2 shouldn't be queued
+        assert!(iter.next().is_none());
+
+        let iter = make_iter(&[id_6.clone(), id_7.clone(), id_2.clone()], &[id_3.clone()]);
+        assert_eq!(iter.walk.queue.items.len(), 2); // id_2 shouldn't be queued
+
+        let iter = make_iter(&[id_6.clone(), id_7.clone()], &[]);
+        assert!(iter.walk.queue.items.is_empty()); // no ids should be queued
+    }
+
+    #[test]
+    fn test_walk_ancestors_filtered_by_generation() {
         let mut new_change_id = change_id_generator();
         let mut index = DefaultMutableIndex::full(3, 16);
         // 8 6
@@ -721,12 +1006,11 @@ mod tests {
 
         let walk_commit_ids = |wanted: &[CommitId], unwanted: &[CommitId], range: Range<u32>| {
             let index = index.as_composite();
-            let wanted_positions = to_positions_vec(index, wanted);
-            let unwanted_positions = to_positions_vec(index, unwanted);
-            index
-                .walk_revs(&wanted_positions, &unwanted_positions)
-                .filter_by_generation(range)
-                .map(|entry| entry.commit_id())
+            RevWalkBuilder::new(index)
+                .wanted_heads(to_positions_vec(index, wanted))
+                .unwanted_roots(to_positions_vec(index, unwanted))
+                .ancestors_filtered_by_generation(range)
+                .map(|pos| index.entry_by_pos(pos).commit_id())
                 .collect_vec()
         };
 
@@ -778,7 +1062,7 @@ mod tests {
 
     #[test]
     #[allow(clippy::redundant_clone)] // allow id_n.clone()
-    fn test_walk_revs_filter_by_generation_range_merging() {
+    fn test_walk_ancestors_filtered_by_generation_range_merging() {
         let mut new_change_id = change_id_generator();
         let mut index = DefaultMutableIndex::full(3, 16);
         // Long linear history with some short branches
@@ -800,11 +1084,10 @@ mod tests {
 
         let walk_commit_ids = |wanted: &[CommitId], range: Range<u32>| {
             let index = index.as_composite();
-            let wanted_positions = to_positions_vec(index, wanted);
-            index
-                .walk_revs(&wanted_positions, &[])
-                .filter_by_generation(range)
-                .map(|entry| entry.commit_id())
+            RevWalkBuilder::new(index)
+                .wanted_heads(to_positions_vec(index, wanted))
+                .ancestors_filtered_by_generation(range)
+                .map(|pos| index.entry_by_pos(pos).commit_id())
                 .collect_vec()
         };
 
@@ -835,7 +1118,7 @@ mod tests {
     }
 
     #[test]
-    fn test_walk_revs_descendants_filtered_by_generation() {
+    fn test_walk_descendants_filtered_by_generation() {
         let mut new_change_id = change_id_generator();
         let mut index = DefaultMutableIndex::full(3, 16);
         // 8 6
@@ -871,12 +1154,10 @@ mod tests {
         let visible_heads = [&id_6, &id_8].map(Clone::clone);
         let walk_commit_ids = |roots: &[CommitId], heads: &[CommitId], range: Range<u32>| {
             let index = index.as_composite();
-            let root_positions = to_positions_vec(index, roots);
-            let head_positions = to_positions_vec(index, heads);
-            index
-                .walk_revs(&head_positions, &[])
-                .descendants_filtered_by_generation(&root_positions, range)
-                .map(|entry| entry.commit_id())
+            RevWalkBuilder::new(index)
+                .wanted_heads(to_positions_vec(index, heads))
+                .descendants_filtered_by_generation(to_positions_vec(index, roots), range)
+                .map(|Reverse(pos)| index.entry_by_pos(pos).commit_id())
                 .collect_vec()
         };
 
