@@ -16,7 +16,7 @@
 
 use std::borrow::Borrow;
 use std::cmp::Ordering;
-use std::fmt::{Debug, Error, Formatter};
+use std::fmt::{self, Debug, Formatter};
 use std::iter::FusedIterator;
 use std::ops::Deref;
 use std::path::{Component, Path, PathBuf};
@@ -28,7 +28,7 @@ use crate::content_hash::ContentHash;
 use crate::file_util;
 
 /// Owned `RepoPath` component.
-#[derive(ContentHash, Clone, PartialEq, Eq, PartialOrd, Ord, Debug, Hash)]
+#[derive(ContentHash, Clone, PartialEq, Eq, PartialOrd, Ord, Hash)]
 pub struct RepoPathComponentBuf {
     // Don't add more fields. Eq, Hash, and Ord must be compatible with the
     // borrowed RepoPathComponent type.
@@ -36,7 +36,7 @@ pub struct RepoPathComponentBuf {
 }
 
 /// Borrowed `RepoPath` component.
-#[derive(PartialEq, Eq, PartialOrd, Ord, Debug, Hash, RefCastCustom)]
+#[derive(PartialEq, Eq, PartialOrd, Ord, Hash, RefCastCustom)]
 #[repr(transparent)]
 pub struct RepoPathComponent {
     value: str,
@@ -59,6 +59,18 @@ impl RepoPathComponent {
     }
 }
 
+impl Debug for RepoPathComponent {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        write!(f, "{:?}", &self.value)
+    }
+}
+
+impl Debug for RepoPathComponentBuf {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        <RepoPathComponent as Debug>::fmt(self, f)
+    }
+}
+
 impl From<&str> for RepoPathComponentBuf {
     fn from(value: &str) -> Self {
         RepoPathComponentBuf::from(value.to_owned())
@@ -172,13 +184,13 @@ pub struct RepoPath {
 }
 
 impl Debug for RepoPath {
-    fn fmt(&self, f: &mut Formatter<'_>) -> Result<(), Error> {
-        f.write_fmt(format_args!("{:?}", &self.value))
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        write!(f, "{:?}", &self.value)
     }
 }
 
 impl Debug for RepoPathBuf {
-    fn fmt(&self, f: &mut Formatter<'_>) -> Result<(), Error> {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
         <RepoPath as Debug>::fmt(self, f)
     }
 }
@@ -440,9 +452,12 @@ pub enum RelativePathParseError {
 #[derive(Clone, Debug, Eq, Error, PartialEq)]
 #[error(r#"Path "{input}" is not in the repo "{base}""#)]
 pub struct FsPathParseError {
-    base: Box<Path>,
-    input: Box<Path>,
-    source: RelativePathParseError,
+    /// Repository or workspace root path relative to the `cwd`.
+    pub base: Box<Path>,
+    /// Input path without normalization.
+    pub input: Box<Path>,
+    /// Source error.
+    pub source: RelativePathParseError,
 }
 
 fn is_valid_repo_path_component_str(value: &str) -> bool {
